@@ -1,8 +1,18 @@
-# Classes and functions
-from .package_info import (
-    VERSION,
-    github,
+# Modules
+#from . import * # Does not work :(
+from . import (
+    lattice,
+    package_info,
+    random,
+    utils,
+    visualization,
+    function_iteration,
+    function_operations,
 )
+
+# Quick access for classes and functions
+from .package_info import (
+    github,)
 from .lattice.lattice import (
     Lattice,
     Poset,
@@ -16,11 +26,3 @@ from .random.random_lattice import (
 )
 
 from .visualization.gui import new_visualizer
-
-# Modules
-from . import (
-    package_info,
-    visualization,
-    function_iteration,
-    function_operations,
-)
